% Example 2 in "Wavelet density estimation for stratified size-biased sample"
% by Pepa Ramirez & Brani Vidakovic. In this example we consider a Beta(2,2) 
% underlying distribution and 3 strata where the biasing functions are 
% g1(y)=y, g2(y)=y^3/2; g3(y)= y^2.
%
%  The program calls to:
%           1) Phijk.m -- Daubechies-Lagarias Algorithm. 
%           2) dwtr.m and idwtr.m -- forward and inverse wavelet transform.
%           3) loc_lin.m and kernel.m -- local linear smoother with a
%           Gaussian kernel.
%
%--------------------------------------------------------------------------
close all
clear all
lw = 2;  
set(0, 'DefaultAxesFontSize', 16);
fs = 14;
msize = 5;
%---- Generate de biased sample size & estimate mu's.
n1 = 500; y1 = betarnd(3,2,n1,1);
n2 = 500; y2 = betarnd(7/2,2,n2,1);
n3 = 500; y3 = betarnd(4,2,n3,1);

%Biasing function: g1(y)=y, g2(y)=y^3/2; g3(y)= y^2.
%Estimatio of mu1,mu2,mu3
muhat1 = n1./sum(1./y1);
muhat2= n2./sum(1./(y2.^(3/2)));
muhat3= n3./sum(1./(y3.^2));

%--------------------------------------------------------------------------
%--- Apply Daubechies Lagarias to obtain the linear wavelet estimate 

% Choose an orthonormal QMF Filter for the D-L algorithm
% for example Symmlet 4:
filter = [-0.07576571478934,  -0.02963552764595,   0.49761866763246,   0.80373875180522,...
   0.29785779560554,  -0.09921954357694,  -0.01260396726226,   0.03222310060407];
J=7; % default coarsest level
cs1 =[];
for k = 0:(2^J-1)
    cJk1 = 0;
    for i = 1:n1
        cJk1 = cJk1 + Phijk(y1(i), J, k, filter, 20)/y1(i); 
        %Evaluation of the scaling function corresponding to an Orthogonal 
        %MRA by Daubechies-Lagarias Algorithm.
    end
    cJk1 = muhat1/(n1+n2+n3) * cJk1;
    cs1 =[cs1 cJk1];
end
cs1 = 2^(J/2) * cs1; % This is the estimate of alpha_1*f^X_1,J in equation (5).

cs2 =[];
for k = 0:(2^J-1)
    cJk2 = 0;
    for i = 1:n2
        cJk2 = cJk2 + Phijk(y2(i), J, k, filter, 20)/(y2(i)^(3/2));
    end
    cJk2 = muhat2/(n1+n2+n3) * cJk2;
    cs2 =[cs2 cJk2];
end
cs2 = 2^(J/2) * cs2; % This is the estimate of alpha_2*f^X_2,J in equation (5).

cs3 =[];
for k = 0:(2^J-1)
    cJk3 = 0;
    for i = 1:n3
        cJk3 = cJk3 + Phijk(y3(i), J, k, filter, 20)/(y3(i)^2);
    end
    cJk3 = muhat3/(n1+n2+n3) * cJk3;
    cs3 =[cs3 cJk3];
end
cs5 = 2^(J/2) * cs3;% This is the estimate of alpha_3*f^X_3,J in equation (7).

% Our linear (or "raw") estimator (equation 7) is
linear_est = cs1 + cs2 + cs3;

%--------------------------------------------------------------------------
% How does the linear estimator look like?
a = linspace(0,1, length(linear_est));
figure(1);
plot( a, 1./beta(2,2).* a .* (1-a),'k-') % underlying unobserved density f^X
hold on
plot( a, 1./beta(3,2).* a.^2 .* (1-a),'k-.') % observed f^Y1
hold on
plot( a, 1./beta(7/2,2).* a.^(5/2) .* (1-a),'k-.') %observed f^Y12
hold on
plot( a, 1./beta(4,2).* a.^3 .* (1-a),'k-.') %observed f^Y13
hold on
plot(a,linear_est,'k:')
%title('Linear wavelet estimator based on projection')
xlabel('y')
ylabel('f(y)')
legend('f^X(y)','f^Y_1(y)','f^Y_2(y)','f^Y_3(y)','f^X_{est}(y)')

%--------------------------------------------------------------------------
% How does the linear estimator look like after hard thresholding?
wf = [0.0386 -0.1270 -0.0772 0.6075 0.7457 0.2266]; % Filter Coiflet(1)
csw = dwtr(linear_est, 6, wf); % Forward wavelet transformation 
nn = length(csw);
nn2 = nn/2;
finest = csw(nn2+1:nn); % finest details.
sigma = 1.4826*median(abs(finest-median(finest)));
lambda = sqrt( 2 * log(nn)) * sigma;
cswt = csw .* ( abs(csw) > lambda); %hard threshold
smooth_th = idwtr(cswt, 6, wf); % Inverse wavelet transformation

figure(2);
plot( a, 1./beta(2,2).* a .* (1-a),'k-') % underlying unobserved density f^X
hold on
plot( a, 1./beta(3,2).* a.^2 .* (1-a),'k-.') % observed f^Y1
hold on
plot( a, 1./beta(7/2,2).* a.^(5/2) .* (1-a),'k-.') %observed f^Y12
hold on
plot( a, 1./beta(4,2).* a.^3 .* (1-a),'k-.') %observed f^Y13
hold on
plot(a, smooth_th,'k--')
%title('Linear wavelet estimator after thresholding')
xlabel('y')
ylabel('f(y)')
legend('f^X(y)','f^Y_1(y)','f^Y_2(y)','f^Y_3(y)','f^X_{est th}(y)')
%--------------------------------------------------------------------------
% How does the linear estimator look like after local linear smoothing?
h = 0.060; % bandwidth
smooth_ll=[];
for j=1:100
  smooth_ll = [smooth_ll loc_lin(j/100, a, linear_est, h)];
end
figure(3)
plot( a, 1./beta(2,2).* a .* (1-a),'k') % underlying unobserved density f^X
hold on
plot( a, 1./beta(3,2).* a.^2 .* (1-a),'k-.') % observed f^Y1
hold on
plot( a, 1./beta(7/2,2).* a.^(5/2) .* (1-a),'k-.') %observed f^Y12
hold on
plot( a, 1./beta(4,2).* a.^3 .* (1-a),'k-.') %observed f^Y13
hold on
plot((1:100)/100, smooth_ll, 'k--')
%title('Smoothed linear wavelet estimator')
xlabel('y')
ylabel('f(y)')
legend('f^X(y)','f^Y_1(y)','f^Y_2(y)','f^Y_3(y)','f^X_{est loc.lin}(y)')

