% Example 3 in "Wavelet density estimation for stratified size-biased sample"
% by Pepa Ramirez & Brani Vidakovic.

% In this example we consider an underlying density with a two triangles
% shape, given by f(x) = 8x, 0<=x<1/4; f(x) = 4(1-2x), 1/4<=x<1/2; f(x) =
% 8x-4, 1/2<=x<3/4; f(x) = 8-8x, 3/4<=x<=1.

% The biasing function is g(y) = y.
%
%  The program calls to:
%           1) Phijk.m -- Daubechies-Lagarias Algorithm. 
%           2) dwtr.m and idwtr.m -- forward and inverse wavelet transform.
%           3) loc_lin.m and kernel.m -- local linear smoother with a
%           Gaussian kernel.
%
%--------------------------------------------------------------------------
close all
clear all
lw = 2;  
set(0, 'DefaultAxesFontSize', 16);
fs = 14;
msize = 5;

%---- Generate de biased sample size & estimate mu.
n = 2000; % we first simulate n observations from the underlying function and
% then we obtain the biased sample (of size less than n). If n = 2000, the
% sample biased size is around 1000.
x = zeros(1,n);
p = unifrnd(0,1,1,n);
i1 = find(p<=0.25);
x(i1) = sqrt(p(i1)/4);
i2 = find(p>0.25 & p<=0.5);
x(i2) = (4-sqrt(8-16*p(i2)))/8;
i3 = find(p>0.5 & p<=0.75);
x(i3) = (4+sqrt(16*p(i3)-8))/8;
i4 = find(p>0.75);
x(i4) = (8-sqrt(16-16*p(i4)))/8;
% Compute the biased sample, using g(y) = y.
y = [];
for i = 1:length(x)
    if unifrnd(0,1)<x(i) % I keep it
        y = [y x(i)];
    end
end
n = length(y);

muhat = n./sum(1./(y));

% To plot the uderlying and observed densities:
xx1 = 0:.001:0.5; 
xx2 = 0.5:.001:1;
a = linspace(0.01,1, 128);
[f1] = ksdensity(y,a); % estimate of the observed density.


%--------------------------------------------------------------------------
%--- Apply Daubechies Lagarias to obtain the linear wavelet estimate (equation 4)

% Choose an orthonormal QMF Filter for the D-L algorithm
% for example Symmlet 4:
filter = [-0.07576571478934,  -0.02963552764595,   0.49761866763246,   0.80373875180522,...
   0.29785779560554,  -0.09921954357694,  -0.01260396726226,   0.03222310060407];
J=7; % default coarsest level
cs =[];
for k = 0:(2^J-1)
    cJk = 0;
    for i = 1:n
        cJk = cJk + Phijk(y(i), J, k, filter, 20)/y(i); 
        %Evaluation of the scaling function corresponding to an Orthogonal 
        %MRA by Daubechies-Lagarias Algorithm.
    end
    cJk = muhat/n * cJk;
    cs =[cs cJk];
end
linear_est = 2^(J/2) * cs; % This is the linear (or "raw") estimator (equation 5)

%--------------------------------------------------------------------------
% How does the linear estimator look like?
a = linspace(0,1, length(linear_est));
figure(1);
plot(xx1(1:251),8*xx1(1:251),'k');
hold on
plot(xx1(252:501),4*(1-2*xx1(252:501)),'k')
hold on
plot(xx2(1:251),8*xx2(1:251)-4,'k');
hold on
plot(xx2(252:501),8-8*xx2(252:501),'k')
plot( a, f1,'k-.') % observed f^Y
hold on
plot(a,linear_est,'k:')
%title('Linear wavelet estimator based on projection')

%--------------------------------------------------------------------------
% How does the linear estimator look like after hard thresholding?

%wf = [0.0386 -0.1270 -0.0772 0.6075 0.7457 0.2266]; % For example, filter
% Coiflet(1). To try another filter: wf = MakeONFilterExt(Type,Par)
wf = [0.3327    0.8069    0.4599   -0.1350   -0.0854    0.0352];%Daubechies 6 
wf = MakeONFilterExt('Daubechies',6);
csw = dwtr(linear_est, 6, wf); % Forward wavelet transformation 
nn = length(csw);
nn2 = nn/2;
finest = csw(nn2+1:nn); %finest details
sigma = 1.4826*median(abs(finest-median(finest)));
lambda = sqrt( 2 * log(nn)) * sigma;
cswt = csw .* ( abs(csw) > lambda); %hard threshold
smooth_th = idwtr(cswt, 6, wf); % Inverse wavelet transformation

figure(2);
plot(xx1(1:251),8*xx1(1:251),'k');
hold on
plot(xx1(252:501),4*(1-2*xx1(252:501)),'k')
hold on
plot(xx2(1:251),8*xx2(1:251)-4,'k');
hold on
plot(xx2(252:501),8-8*xx2(252:501),'k')
plot( a, f1,'k-.') % observed f^Y
hold on
plot(a, smooth_th,'k--')
%title('Linear wavelet estimator after thresholding')
%--------------------------------------------------------------------------

% How does it look like after local linear smoothing?
h = 0.080; % bandwidth
smooth_ll=[];
for j=1:100
  smooth_ll = [smooth_ll loc_lin(j/100, a, linear_est, h)];
end
figure(3)
plot(xx1(1:251),8*xx1(1:251),'k');
hold on
plot(xx1(252:501),4*(1-2*xx1(252:501)),'k')
hold on
plot(xx2(1:251),8*xx2(1:251)-4,'k');
hold on
plot(xx2(252:501),8-8*xx2(252:501),'k')
plot( a, f1,'k-.') % observed f^Y
hold on
plot((1:100)/100, smooth_ll, 'k--')
%title('Smoothed linear wavelet estimator')


